@@ -1,6 +1,7 @@
-# -*- coding: us-ascii -*-
 require 'rubygems/test_case'
+require 'pathname'
 require 'stringio'
+require 'rubygems/ext'
 require 'rubygems/specification'
 
 class TestGemSpecification < Gem::TestCase
@@ -39,7 +40,7 @@ end
   EOF
 
   def make_spec_c1
-    @c1 = quick_spec 'a', '1' do |s|
+    @c1 = util_spec 'a', '1' do |s|
       s.executable = 'exec'
       s.extensions << 'ext/a/extconf.rb'
       s.test_file = 'test/suite.rb'
@@ -56,12 +57,24 @@ end
     end
   end
 
+  def ext_spec
+    @ext = util_spec 'ext', '1' do |s|
+      s.executable = 'exec'
+      s.test_file = 'test/suite.rb'
+      s.extensions = %w[ext/extconf.rb]
+      s.license = 'MIT'
+
+      s.mark_version
+      s.files = %w[lib/code.rb]
+      s.installed_by_version = v('2.2')
+    end
+  end
+
   def setup
     super
 
-    @a1 = quick_spec 'a', '1' do |s|
+    @a1 = util_spec 'a', '1' do |s|
       s.executable = 'exec'
-      s.extensions << 'ext/a/extconf.rb'
       s.test_file = 'test/suite.rb'
       s.requirements << 'A working computer'
       s.rubyforge_project = 'example'
@@ -71,11 +84,11 @@ end
       s.files = %w[lib/code.rb]
     end
 
-    @a2 = quick_spec 'a', '2' do |s|
+    @a2 = util_spec 'a', '2' do |s|
       s.files = %w[lib/code.rb]
     end
 
-    @a3 = quick_spec 'a', '3' do |s|
+    @a3 = util_spec 'a', '3' do |s|
       s.metadata['allowed_push_host'] = "https://privategemserver.com"
     end
 
@@ -212,7 +225,7 @@ end
             util_spec 'b', '2.0'
     c,  _ = util_spec 'c', '1.0', 'b' => '= 2.0'
 
-    e = assert_raises Gem::LoadError do
+    e = assert_raises Gem::ConflictError do
       assert_activate nil, a, c, "b"
     end
 
@@ -702,6 +715,20 @@ dependencies: []
     assert_equal @a2, spec
   end
 
+  def test_self_load_relative
+    open 'a-2.gemspec', 'w' do |io|
+      io.write @a2.to_ruby_for_cache
+    end
+
+    spec = Gem::Specification.load 'a-2.gemspec'
+
+    @a2.files.clear
+
+    assert_equal @a2, spec
+
+    assert_equal File.join(@tempdir, 'a-2.gemspec'), spec.loaded_from
+  end
+
   def test_self_load_tainted
     full_path = @a2.spec_file
     write_file full_path do |io|
@@ -826,37 +853,59 @@ dependencies: []
   end
 
   def test_self_outdated
-    util_clear_gems
-    util_setup_fake_fetcher true
+    spec_fetcher do |fetcher|
+      fetcher.spec 'a', 4
 
-    a4 = quick_gem @a1.name, '4'
-    util_build_gem a4
-    util_setup_spec_fetcher @a1, @a2, @a3a, a4
+      fetcher.clear
 
-    Gem::Specification.remove_spec @a1
-    Gem::Specification.remove_spec @a2
-    Gem::Specification.remove_spec a4
+      fetcher.spec 'a', 3
+    end
 
     assert_equal %w[a], Gem::Specification.outdated
   end
 
   def test_self_outdated_and_latest_remotes
-    util_clear_gems
-    util_setup_fake_fetcher true
+    specs = spec_fetcher do |fetcher|
+      fetcher.spec 'a', 4
+      fetcher.spec 'b', 3
 
-    a4 = quick_gem @a1.name, '4'
-    util_build_gem a4
-    b3 = quick_gem @b2.name, '3'
-    util_build_gem b3
-    util_setup_spec_fetcher @a1, @a2, @a3a, a4, @b2, b3
+      fetcher.clear
+
+      fetcher.spec 'a', '3.a'
+      fetcher.spec 'b', 2
+    end
+
+    expected = [
+      [specs['a-3.a'], v(4)],
+      [specs['b-2'],   v(3)],
+    ]
+
+    assert_equal expected, Gem::Specification.outdated_and_latest_version.to_a
+  end
+
+  def test_self_remove_spec
+    assert_includes Gem::Specification.all_names, 'a-1'
+    assert_includes Gem::Specification.stubs.map { |s| s.full_name }, 'a-1'
 
     Gem::Specification.remove_spec @a1
-    Gem::Specification.remove_spec @a2
-    Gem::Specification.remove_spec a4
-    Gem::Specification.remove_spec b3
 
-    assert_equal [[@a3a, a4.version], [@b2, b3.version]],
-                 Gem::Specification.outdated_and_latest_version.to_a
+    refute_includes Gem::Specification.all_names, 'a-1'
+    refute_includes Gem::Specification.stubs.map { |s| s.full_name }, 'a-1'
+  end
+
+  def test_self_remove_spec_removed
+    open @a1.spec_file, 'w' do |io|
+      io.write @a1.to_ruby
+    end
+
+    Gem::Specification.reset
+
+    FileUtils.rm @a1.spec_file # bug #698
+
+    Gem::Specification.remove_spec @a1
+
+    refute_includes Gem::Specification.all_names, 'a-1'
+    refute_includes Gem::Specification.stubs.map { |s| s.full_name }, 'a-1'
   end
 
   DATA_PATH = File.expand_path "../data", __FILE__
@@ -949,8 +998,8 @@ dependencies: []
     assert_equal 'summary', spec.summary
     assert_same spec.summary, new_spec.summary
 
-    assert_equal %w[lib/file.rb test/file.rb bin/exec README.txt
-                    ext/extconf.rb].sort,
+    assert_equal %w[README.txt bin/exec ext/extconf.rb lib/file.rb
+                    test/file.rb].sort,
                  spec.files
     refute_same spec.files, new_spec.files, 'files'
 
@@ -1023,7 +1072,7 @@ dependencies: []
   end
 
   def test_add_dependency_with_type
-    gem = quick_spec "awesome", "1.0" do |awesome|
+    gem = util_spec "awesome", "1.0" do |awesome|
       awesome.add_dependency true
       awesome.add_dependency :gem_name
     end
@@ -1032,7 +1081,7 @@ dependencies: []
   end
 
   def test_add_dependency_with_type_explicit
-    gem = quick_spec "awesome", "1.0" do |awesome|
+    gem = util_spec "awesome", "1.0" do |awesome|
       awesome.add_development_dependency "monkey"
     end
 
@@ -1059,7 +1108,254 @@ dependencies: []
     @a2.executable = 'app'
 
     assert_equal nil, @a2.bindir
-    assert_equal %w[lib/code.rb app].sort, @a2.files
+    assert_equal %w[app lib/code.rb].sort, @a2.files
+  end
+
+  def test_extensions_equals_nil
+    @a2.instance_variable_set(:@extensions, nil)
+    assert_equal nil, @a2.instance_variable_get(:@extensions)
+    assert_equal %w[lib/code.rb], @a2.files
+  end
+
+  def test_test_files_equals_nil
+    @a2.instance_variable_set(:@test_files, nil)
+    assert_equal nil, @a2.instance_variable_get(:@test_files)
+    assert_equal %w[lib/code.rb], @a2.files
+  end
+
+  def test_executables_equals_nil
+    @a2.instance_variable_set(:@executables, nil)
+    assert_equal nil, @a2.instance_variable_get(:@executables)
+    assert_equal %w[lib/code.rb], @a2.files
+  end
+
+  def test_extra_rdoc_files_equals_nil
+    @a2.instance_variable_set(:@extra_rdoc_files, nil)
+    assert_equal nil, @a2.instance_variable_get(:@extra_rdoc_files)
+    assert_equal %w[lib/code.rb], @a2.files
+  end
+
+  def test_build_args
+    ext_spec
+
+    assert_empty @ext.build_args
+
+    open @ext.build_info_file, 'w' do |io|
+      io.puts
+    end
+
+    assert_empty @ext.build_args
+
+    open @ext.build_info_file, 'w' do |io|
+      io.puts '--with-foo-dir=wherever'
+    end
+
+    assert_equal %w[--with-foo-dir=wherever], @ext.build_args
+  end
+
+  def test_build_extensions
+    ext_spec
+
+    refute_path_exists @ext.extension_dir, 'sanity check'
+    refute_empty @ext.extensions, 'sanity check'
+
+    extconf_rb = File.join @ext.gem_dir, @ext.extensions.first
+    FileUtils.mkdir_p File.dirname extconf_rb
+
+    open extconf_rb, 'w' do |f|
+      f.write <<-'RUBY'
+        open 'Makefile', 'w' do |f|
+          f.puts "clean:\n\techo clean"
+          f.puts "default:\n\techo built"
+          f.puts "install:\n\techo installed"
+        end
+      RUBY
+    end
+
+    @ext.build_extensions
+
+    assert_path_exists @ext.extension_dir
+  end
+
+  def test_build_extensions_built
+    ext_spec
+
+    refute_empty @ext.extensions, 'sanity check'
+
+    gem_build_complete =
+      File.join @ext.extension_dir, 'gem.build_complete'
+
+    FileUtils.mkdir_p @ext.extension_dir
+    FileUtils.touch gem_build_complete
+
+    @ext.build_extensions
+
+    gem_make_out = File.join @ext.extension_dir, 'gem_make.out'
+    refute_path_exists gem_make_out
+  end
+
+  def test_build_extensions_default_gem
+    spec = new_default_spec 'default', 1
+    spec.extensions << 'extconf.rb'
+
+    extconf_rb = File.join spec.gem_dir, spec.extensions.first
+    FileUtils.mkdir_p File.dirname extconf_rb
+
+    open extconf_rb, 'w' do |f|
+      f.write <<-'RUBY'
+        open 'Makefile', 'w' do |f|
+          f.puts "default:\n\techo built"
+          f.puts "install:\n\techo installed"
+        end
+      RUBY
+    end
+
+    spec.build_extensions
+
+    refute_path_exists spec.extension_dir
+  end
+
+  def test_build_extensions_error
+    ext_spec
+
+    refute_empty @ext.extensions, 'sanity check'
+
+    assert_raises Gem::Ext::BuildError do
+      @ext.build_extensions
+    end
+  end
+
+  def test_build_extensions_extensions_dir_unwritable
+    skip 'chmod not supported' if Gem.win_platform?
+
+    ext_spec
+
+    refute_empty @ext.extensions, 'sanity check'
+
+    extconf_rb = File.join @ext.gem_dir, @ext.extensions.first
+    FileUtils.mkdir_p File.dirname extconf_rb
+
+    open extconf_rb, 'w' do |f|
+      f.write <<-'RUBY'
+        open 'Makefile', 'w' do |f|
+          f.puts "clean:\n\techo clean"
+          f.puts "default:\n\techo built"
+          f.puts "install:\n\techo installed"
+        end
+      RUBY
+    end
+
+    FileUtils.mkdir_p File.join @ext.base_dir, 'extensions'
+    FileUtils.chmod 0555, @ext.base_dir
+    FileUtils.chmod 0555, File.join(@ext.base_dir, 'extensions')
+
+    @ext.build_extensions
+    refute_path_exists @ext.extension_dir
+  ensure
+    unless ($DEBUG or win_platform?) then
+      FileUtils.chmod 0755, File.join(@ext.base_dir, 'extensions')
+      FileUtils.chmod 0755, @ext.base_dir
+    end
+  end
+
+  def test_build_extensions_no_extensions_dir_unwritable
+    skip 'chmod not supported' if Gem.win_platform?
+
+    ext_spec
+
+    refute_empty @ext.extensions, 'sanity check'
+
+    extconf_rb = File.join @ext.gem_dir, @ext.extensions.first
+    FileUtils.mkdir_p File.dirname extconf_rb
+
+    open extconf_rb, 'w' do |f|
+      f.write <<-'RUBY'
+        open 'Makefile', 'w' do |f|
+          f.puts "clean:\n\techo clean"
+          f.puts "default:\n\techo built"
+          f.puts "install:\n\techo installed"
+        end
+      RUBY
+    end
+
+    FileUtils.rm_r File.join @gemhome, 'extensions'
+    FileUtils.chmod 0555, @gemhome
+
+    @ext.build_extensions
+
+    gem_make_out = File.join @ext.extension_dir, 'gem_make.out'
+    refute_path_exists gem_make_out
+  ensure
+    FileUtils.chmod 0755, @gemhome
+  end
+
+  def test_build_extensions_none
+    refute_path_exists @a1.extension_dir, 'sanity check'
+    assert_empty @a1.extensions, 'sanity check'
+
+    @a1.build_extensions
+
+    refute_path_exists @a1.extension_dir
+  end
+
+  def test_build_extensions_old
+    ext_spec
+
+    refute_empty @ext.extensions, 'sanity check'
+
+    @ext.installed_by_version = v(0)
+
+    @ext.build_extensions
+
+    gem_make_out = File.join @ext.extension_dir, 'gem_make.out'
+    refute_path_exists gem_make_out
+  end
+
+  def test_build_extensions_preview
+    ext_spec
+
+    extconf_rb = File.join @ext.gem_dir, @ext.extensions.first
+    FileUtils.mkdir_p File.dirname extconf_rb
+
+    open extconf_rb, 'w' do |f|
+      f.write <<-'RUBY'
+        open 'Makefile', 'w' do |f|
+          f.puts "clean:\n\techo clean"
+          f.puts "default:\n\techo built"
+          f.puts "install:\n\techo installed"
+        end
+      RUBY
+    end
+
+    refute_empty @ext.extensions, 'sanity check'
+
+    @ext.installed_by_version = v('2.2.0.preview.2')
+
+    @ext.build_extensions
+
+    gem_make_out = File.join @ext.extension_dir, 'gem_make.out'
+    assert_path_exists gem_make_out
+  end
+
+  def test_contains_requirable_file_eh
+    code_rb = File.join @a1.gem_dir, 'lib', 'code.rb'
+    FileUtils.mkdir_p File.dirname code_rb
+    FileUtils.touch code_rb
+
+    assert @a1.contains_requirable_file? 'code'
+  end
+
+  def test_contains_requirable_file_eh_extension
+    ext_spec
+
+    _, err = capture_io do
+      refute @ext.contains_requirable_file? 'nonexistent'
+    end
+
+    expected = "Ignoring ext-1 because its extensions are not built.  " +
+               "Try: gem pristine ext-1\n"
+
+    assert_equal expected, err
   end
 
   def test_date
@@ -1112,6 +1408,20 @@ dependencies: []
     assert_equal [@bonobo, @monkey], @gem.dependencies
   end
 
+  def test_dependent_gems
+    util_setup_deps
+
+    assert_empty @gem.dependent_gems
+
+    bonobo = util_spec 'bonobo'
+
+    expected = [
+      [@gem, @bonobo, [bonobo]],
+    ]
+
+    assert_equal expected, bonobo.dependent_gems
+  end
+
   def test_doc_dir
     assert_equal File.join(@gemhome, 'doc', 'a-1'), @a1.doc_dir
   end
@@ -1159,11 +1469,57 @@ dependencies: []
   def test_executable_equals
     @a2.executable = 'app'
     assert_equal 'app', @a2.executable
-    assert_equal %w[lib/code.rb bin/app].sort, @a2.files
+    assert_equal %w[bin/app lib/code.rb].sort, @a2.files
   end
 
   def test_extensions
-    assert_equal ['ext/a/extconf.rb'], @a1.extensions
+    assert_equal ['ext/extconf.rb'], ext_spec.extensions
+  end
+
+  def test_extension_dir
+    enable_shared, RbConfig::CONFIG['ENABLE_SHARED'] =
+      RbConfig::CONFIG['ENABLE_SHARED'], 'no'
+
+    ext_spec
+
+    refute_empty @ext.extensions
+
+    expected =
+      File.join(@ext.base_dir, 'extensions', Gem::Platform.local.to_s,
+                "#{Gem.ruby_api_version}-static", @ext.full_name)
+
+    assert_equal expected, @ext.extension_dir
+  ensure
+    RbConfig::CONFIG['ENABLE_SHARED'] = enable_shared
+  end
+
+  def test_extension_dir_override
+    enable_shared, RbConfig::CONFIG['ENABLE_SHARED'] =
+      RbConfig::CONFIG['ENABLE_SHARED'], 'no'
+
+    class << Gem
+      alias orig_default_ext_dir_for default_ext_dir_for
+
+      def Gem.default_ext_dir_for(base_dir)
+        'elsewhere'
+      end
+    end
+
+    ext_spec
+
+    refute_empty @ext.extensions
+
+    expected = File.join @tempdir, 'elsewhere', @ext.full_name
+
+    assert_equal expected, @ext.extension_dir
+  ensure
+    RbConfig::CONFIG['ENABLE_SHARED'] = enable_shared
+
+    class << Gem
+      remove_method :default_ext_dir_for
+
+      alias default_ext_dir_for orig_default_ext_dir_for
+    end
   end
 
   def test_files
@@ -1314,12 +1670,25 @@ dependencies: []
     end
   end
 
+  def test_gem_build_complete_path
+    expected = File.join @a1.extension_dir, 'gem.build_complete'
+    assert_equal expected, @a1.gem_build_complete_path
+  end
+
   def test_hash
     assert_equal @a1.hash, @a1.hash
     assert_equal @a1.hash, @a1.dup.hash
     refute_equal @a1.hash, @a2.hash
   end
 
+  def test_installed_by_version
+    assert_equal v(0), @a1.installed_by_version
+
+    @a1.installed_by_version = Gem.rubygems_version
+
+    assert_equal Gem.rubygems_version, @a1.installed_by_version
+  end
+
   def test_base_dir
     assert_equal @gemhome, @a1.base_dir
   end
@@ -1422,20 +1791,76 @@ dependencies: []
   end
 
   def test_prerelease_spec_adds_required_rubygems_version
-    @prerelease = quick_spec('tardis', '2.2.0.a')
+    @prerelease = util_spec('tardis', '2.2.0.a')
     refute @prerelease.required_rubygems_version.satisfied_by?(Gem::Version.new('1.3.1'))
     assert @prerelease.required_rubygems_version.satisfied_by?(Gem::Version.new('1.4.0'))
   end
 
   def test_require_paths
-    @a1.require_path = 'lib'
-    assert_equal %w[lib], @a1.require_paths
+    enable_shared 'no' do
+      ext_spec
+
+      @ext.require_paths = 'lib'
+
+      assert_equal [@ext.extension_dir, 'lib'], @ext.require_paths
+    end
+  end
+
+  def test_require_paths_default_ext_dir_for
+    class << Gem
+      send :alias_method, :orig_default_ext_dir_for, :default_ext_dir_for
+    end
+
+    def Gem.default_ext_dir_for base_dir
+      '/foo'
+    end
+
+    enable_shared 'no' do
+      ext_spec
+
+      @ext.require_paths = 'lib'
+
+      assert_equal ['/foo/ext-1', 'lib'], @ext.require_paths
+    end
+  ensure
+    class << Gem
+      send :remove_method, :default_ext_dir_for
+      send :alias_method,  :default_ext_dir_for, :orig_default_ext_dir_for
+      send :remove_method, :orig_default_ext_dir_for
+    end
+  end
+
+  def test_source
+    assert_kind_of Gem::Source::Installed, @a1.source
+  end
+
+  def test_source_paths
+    ext_spec
+
+    @ext.require_paths = %w[lib ext foo]
+    @ext.extensions << 'bar/baz'
+
+    expected = %w[
+      lib
+      ext
+      foo
+      bar
+    ]
+
+    assert_equal expected, @ext.source_paths
   end
 
   def test_full_require_paths
-    @a1.require_path = 'lib'
-    assert_equal [File.join(@gemhome, 'gems', @a1.original_name, 'lib')],
-                 @a1.full_require_paths
+    ext_spec
+
+    @ext.require_paths = 'lib'
+
+    expected = [
+      @ext.extension_dir,
+      File.join(@gemhome, 'gems', @ext.original_name, 'lib'),
+    ]
+
+    assert_equal expected, @ext.full_require_paths
   end
 
   def test_require_already_activated
@@ -1550,26 +1975,26 @@ dependencies: []
     @a2.add_runtime_dependency 'b', '1'
     @a2.dependencies.first.instance_variable_set :@type, nil
     @a2.required_rubygems_version = Gem::Requirement.new '> 0'
-    @a2.require_paths << "lib/a/ext"
+    @a2.require_paths << 'other'
 
     ruby_code = @a2.to_ruby
 
     expected = <<-SPEC
 # -*- encoding: utf-8 -*-
-# stub: a 2 ruby lib\0lib/a/ext
+# stub: a 2 ruby lib\0other
 
 Gem::Specification.new do |s|
   s.name = "a"
   s.version = "2"
 
   s.required_rubygems_version = Gem::Requirement.new(\"> 0\") if s.respond_to? :required_rubygems_version=
+  s.require_paths = ["lib", "other"]
   s.authors = ["A User"]
   s.date = "#{Gem::Specification::TODAY.strftime "%Y-%m-%d"}"
   s.description = "This is a test description"
   s.email = "example@example.com"
   s.files = ["lib/code.rb"]
   s.homepage = "http://example.com"
-  s.require_paths = ["lib", "lib/a/ext"]
   s.rubygems_version = "#{Gem::VERSION}"
   s.summary = "this is a summary"
 
@@ -1598,6 +2023,7 @@ end
     @a2.add_runtime_dependency 'b', '1'
     @a2.dependencies.first.instance_variable_set :@type, nil
     @a2.required_rubygems_version = Gem::Requirement.new '> 0'
+    @a2.installed_by_version = Gem.rubygems_version
 
     # cached specs do not have spec.files populated:
     ruby_code = @a2.to_ruby_for_cache
@@ -1611,15 +2037,17 @@ Gem::Specification.new do |s|
   s.version = "2"
 
   s.required_rubygems_version = Gem::Requirement.new(\"> 0\") if s.respond_to? :required_rubygems_version=
+  s.require_paths = ["lib"]
   s.authors = ["A User"]
   s.date = "#{Gem::Specification::TODAY.strftime "%Y-%m-%d"}"
   s.description = "This is a test description"
   s.email = "example@example.com"
   s.homepage = "http://example.com"
-  s.require_paths = ["lib"]
   s.rubygems_version = "#{Gem::VERSION}"
   s.summary = "this is a summary"
 
+  s.installed_by_version = "#{Gem::VERSION}" if s.respond_to? :installed_by_version
+
   if s.respond_to? :specification_version then
     s.specification_version = #{Gem::Specification::CURRENT_SPECIFICATION_VERSION}
 
@@ -1651,10 +2079,14 @@ end
 
     local = Gem::Platform.local
     expected_platform = "[#{local.cpu.inspect}, #{local.os.inspect}, #{local.version.inspect}]"
+    stub_require_paths =
+      @c1.instance_variable_get(:@require_paths).join "\u0000"
+    extensions = @c1.extensions.join "\u0000"
 
     expected = <<-SPEC
 # -*- encoding: utf-8 -*-
-# stub: a 1 #{win_platform? ? "x86-mswin32-60" : "x86-darwin-8"} lib
+# stub: a 1 #{win_platform? ? "x86-mswin32-60" : "x86-darwin-8"} #{stub_require_paths}
+# stub: #{extensions}
 
 Gem::Specification.new do |s|
   s.name = "a"
@@ -1662,6 +2094,7 @@ Gem::Specification.new do |s|
   s.platform = Gem::Platform.new(#{expected_platform})
 
   s.required_rubygems_version = Gem::Requirement.new(\">= 0\") if s.respond_to? :required_rubygems_version=
+  s.require_paths = ["lib"]
   s.authors = ["A User"]
   s.date = "#{Gem::Specification::TODAY.strftime "%Y-%m-%d"}"
   s.description = "This is a test description"
@@ -1671,7 +2104,6 @@ Gem::Specification.new do |s|
   s.files = ["bin/exec", "ext/a/extconf.rb", "lib/code.rb", "test/suite.rb"]
   s.homepage = "http://example.com"
   s.licenses = ["MIT"]
-  s.require_paths = ["lib"]
   s.requirements = ["A working computer"]
   s.rubyforge_project = "example"
   s.rubygems_version = "#{Gem::VERSION}"
@@ -1805,7 +2237,7 @@ end
         @a1.validate
       end
 
-      assert_equal "#{w}:  no author specified\n", @ui.error, 'error'
+      assert_match "#{w}:  no author specified\n", @ui.error, 'error'
 
       @a1.authors = [Object.new]
 
@@ -1845,7 +2277,7 @@ end
         @a1.validate
       end
 
-      assert_equal "#{w}:  deprecated autorequire specified\n",
+      assert_match "#{w}:  deprecated autorequire specified\n",
                    @ui.error, 'error'
     end
   end
@@ -1856,6 +2288,16 @@ end
     Dir.chdir @tempdir do
       @a1.add_runtime_dependency     'b', '>= 1.0.rc1'
       @a1.add_development_dependency 'c', '>= 2.0.rc2'
+      @a1.add_runtime_dependency     'd', '~> 1.2.3'
+      @a1.add_runtime_dependency     'e', '~> 1.2.3.4'
+      @a1.add_runtime_dependency     'g', '~> 1.2.3', '>= 1.2.3.4'
+      @a1.add_runtime_dependency     'h', '>= 1.2.3', '<= 2'
+      @a1.add_runtime_dependency     'i', '>= 1.2'
+      @a1.add_runtime_dependency     'j', '>= 1.2.3'
+      @a1.add_runtime_dependency     'k', '> 1.2'
+      @a1.add_runtime_dependency     'l', '> 1.2.3'
+      @a1.add_runtime_dependency     'm', '~> 2.1.0'
+      @a1.add_runtime_dependency     'n', '~> 0.1.0'
 
       use_ui @ui do
         @a1.validate
@@ -1864,12 +2306,60 @@ end
       expected = <<-EXPECTED
 #{w}:  prerelease dependency on b (>= 1.0.rc1) is not recommended
 #{w}:  prerelease dependency on c (>= 2.0.rc2, development) is not recommended
+#{w}:  pessimistic dependency on d (~> 1.2.3) may be overly strict
+  if d is semantically versioned, use:
+    add_runtime_dependency 'd', '~> 1.2', '>= 1.2.3'
+#{w}:  pessimistic dependency on e (~> 1.2.3.4) may be overly strict
+  if e is semantically versioned, use:
+    add_runtime_dependency 'e', '~> 1.2', '>= 1.2.3.4'
+#{w}:  open-ended dependency on i (>= 1.2) is not recommended
+  if i is semantically versioned, use:
+    add_runtime_dependency 'i', '~> 1.2'
+#{w}:  open-ended dependency on j (>= 1.2.3) is not recommended
+  if j is semantically versioned, use:
+    add_runtime_dependency 'j', '~> 1.2', '>= 1.2.3'
+#{w}:  open-ended dependency on k (> 1.2) is not recommended
+  if k is semantically versioned, use:
+    add_runtime_dependency 'k', '~> 1.2', '> 1.2'
+#{w}:  open-ended dependency on l (> 1.2.3) is not recommended
+  if l is semantically versioned, use:
+    add_runtime_dependency 'l', '~> 1.2', '> 1.2.3'
+#{w}:  pessimistic dependency on m (~> 2.1.0) may be overly strict
+  if m is semantically versioned, use:
+    add_runtime_dependency 'm', '~> 2.1', '>= 2.1.0'
+#{w}:  See http://guides.rubygems.org/specification-reference/ for help
       EXPECTED
 
       assert_equal expected, @ui.error, 'warning'
     end
   end
 
+  def test_validate_dependencies_open_ended
+    util_setup_validate
+
+    Dir.chdir @tempdir do
+      @a1.add_runtime_dependency 'b', '~> 1.2'
+      @a1.add_runtime_dependency 'b', '>= 1.2.3'
+
+      use_ui @ui do
+        e = assert_raises Gem::InvalidSpecificationException do
+          @a1.validate
+        end
+
+        expected = <<-EXPECTED
+duplicate dependency on b (>= 1.2.3), (~> 1.2) use:
+    add_runtime_dependency 'b', '>= 1.2.3', '~> 1.2'
+        EXPECTED
+
+        assert_equal expected, e.message
+      end
+
+      assert_equal <<-EXPECTED, @ui.error
+#{w}:  See http://guides.rubygems.org/specification-reference/ for help
+      EXPECTED
+    end
+  end
+
   def test_validate_description
     util_setup_validate
 
@@ -1880,7 +2370,7 @@ end
         @a1.validate
       end
 
-      assert_equal "#{w}:  no description specified\n", @ui.error, "error"
+      assert_match "#{w}:  no description specified\n", @ui.error, "error"
 
       @ui = Gem::MockGemUi.new
       @a1.summary = "this is my summary"
@@ -1890,7 +2380,7 @@ end
         @a1.validate
       end
 
-      assert_equal "#{w}:  description and summary are identical\n",
+      assert_match "#{w}:  description and summary are identical\n",
                    @ui.error, "error"
 
       @a1.description = "#{f} (describe your package)"
@@ -1921,7 +2411,7 @@ end
         @a1.validate
       end
 
-      assert_equal "#{w}:  no email specified\n", @ui.error, "error"
+      assert_match "#{w}:  no email specified\n", @ui.error, "error"
 
       @a1.email = "FIxxxXME (your e-mail)".sub(/xxx/, "")
 
@@ -1949,6 +2439,16 @@ end
     assert_equal 'missing value for attribute name', e.message
   end
 
+  def test_validate_error
+    assert_raises Gem::InvalidSpecificationException do
+      use_ui @ui do
+        Gem::Specification.new.validate
+      end
+    end
+
+    assert_match 'See http://guides.rubygems.org/specification-reference/ for help', @ui.error
+  end
+
   def test_validate_executables
     util_setup_validate
 
@@ -1965,7 +2465,7 @@ end
     assert_equal %w[exec], @a1.executables
 
     assert_equal '', @ui.output, 'output'
-    assert_equal "#{w}:  bin/exec is missing #! line\n", @ui.error, 'error'
+    assert_match "#{w}:  bin/exec is missing #! line\n", @ui.error, 'error'
   end
 
   def test_validate_empty_require_paths
@@ -1989,6 +2489,7 @@ end
     util_setup_validate
 
     @a1.files += ['lib', 'lib2']
+    @a1.extensions << 'ext/a/extconf.rb'
 
     Dir.chdir @tempdir do
       FileUtils.ln_s '/root/path', 'lib2' unless vc_windows?
@@ -2000,7 +2501,7 @@ end
       assert_equal '["lib2"] are not files', e.message
     end
 
-    assert_equal %w[lib/code.rb test/suite.rb bin/exec ext/a/extconf.rb lib2].sort,
+    assert_equal %w[bin/exec ext/a/extconf.rb lib/code.rb lib2 test/suite.rb].sort,
                  @a1.files
   end
 
@@ -2028,7 +2529,7 @@ end
         @a1.validate
       end
 
-      assert_equal "#{w}:  no homepage specified\n", @ui.error, 'error'
+      assert_match "#{w}:  no homepage specified\n", @ui.error, 'error'
 
       @ui = Gem::MockGemUi.new
 
@@ -2038,7 +2539,7 @@ end
         @a1.validate
       end
 
-      assert_equal "#{w}:  no homepage specified\n", @ui.error, 'error'
+      assert_match "#{w}:  no homepage specified\n", @ui.error, 'error'
 
       @a1.homepage = 'over at my cool site'
 
@@ -2050,6 +2551,20 @@ end
     end
   end
 
+  def test_validate_license
+    util_setup_validate
+
+    use_ui @ui do
+      @a1.licenses.clear
+      @a1.validate
+    end
+
+    assert_match <<-warning, @ui.error
+WARNING:  licenses is empty, but is recommended.  Use a license abbreviation from:
+http://opensource.org/licenses/alphabetical
+    warning
+  end
+
   def test_validate_name
     util_setup_validate
 
@@ -2154,7 +2669,7 @@ end
         @a1.validate
       end
 
-      assert_equal "#{w}:  no summary specified\n", @ui.error, 'error'
+      assert_match "#{w}:  no summary specified\n", @ui.error, 'error'
 
       @a1.summary = "#{f} (describe your package)"
 
@@ -2174,6 +2689,17 @@ end
     end
   end
 
+  def test_validate_warning
+    util_setup_validate
+
+    use_ui @ui do
+      @a1.licenses.clear
+      @a1.validate
+    end
+
+    assert_match 'See http://guides.rubygems.org/specification-reference/ for help', @ui.error
+  end
+
   def test_version
     assert_equal Gem::Version.new('1'), @a1.version
   end
@@ -2233,48 +2759,30 @@ end
   # KEEP a-3-x86-other_platform-1
 
   def test_latest_specs
-    util_clear_gems
-    util_setup_fake_fetcher
+    spec_fetcher do |fetcher|
+      fetcher.spec 'a', 1 do |s|
+        s.platform = Gem::Platform.new 'x86-my_platform1'
+      end
 
-    quick_spec 'p', '1'
+      fetcher.spec 'a', 2
 
-    p1_curr = quick_spec 'p', '1' do |spec|
-      spec.platform = Gem::Platform::CURRENT
+      fetcher.spec 'a', 2 do |s|
+        s.platform = Gem::Platform.new 'x86-my_platform1'
+      end
+
+      fetcher.spec 'a', 2 do |s|
+        s.platform = Gem::Platform.new 'x86-other_platform1'
+      end
+
+      fetcher.spec 'a', 3 do |s|
+        s.platform = Gem::Platform.new 'x86-other_platform1'
+      end
     end
 
-    quick_spec @a1.name, @a1.version do |s|
-      s.platform = Gem::Platform.new 'x86-my_platform1'
-    end
-
-    quick_spec @a1.name, @a1.version do |s|
-      s.platform = Gem::Platform.new 'x86-third_platform1'
-    end
-
-    quick_spec @a2.name, @a2.version do |s|
-      s.platform = Gem::Platform.new 'x86-my_platform1'
-    end
-
-    quick_spec @a2.name, @a2.version do |s|
-      s.platform = Gem::Platform.new 'x86-other_platform1'
-    end
-
-    quick_spec @a2.name, @a2.version.bump do |s|
-      s.platform = Gem::Platform.new 'x86-other_platform1'
-    end
-
-    Gem::Specification.remove_spec @b2
-    Gem::Specification.remove_spec @pl1
-
     expected = %W[
                   a-2
                   a-2-x86-my_platform-1
                   a-3-x86-other_platform-1
-                  a_evil-9
-                  c-1.2
-                  dep_x-1
-                  p-1
-                  #{p1_curr.full_name}
-                  x-1
                  ]
 
     latest_specs = Gem::Specification.latest_specs.map(&:full_name).sort
@@ -2376,13 +2884,13 @@ Gem::Specification.new do |s|
 
   s.required_rubygems_version = Gem::Requirement.new(">= 0") if s.respond_to? :required_rubygems_version=
   s.metadata = { "one" => "two", "two" => "three" } if s.respond_to? :metadata=
+  s.require_paths = ["lib"]
   s.authors = ["A User"]
   s.date = "#{Gem::Specification::TODAY.strftime("%Y-%m-%d")}"
   s.description = "This is a test description"
   s.email = "example@example.com"
   s.files = ["lib/code.rb"]
   s.homepage = "http://example.com"
-  s.require_paths = ["lib"]
   s.rubygems_version = "#{Gem::VERSION}"
   s.summary = "this is a summary"
 end
@@ -2396,14 +2904,76 @@ end
     assert_equal @m1.to_ruby, valid_ruby_spec
   end
 
-  def test_find_by_name
-    util_make_gems
-    assert(Gem::Specification.find_by_name("a"))
-    assert(Gem::Specification.find_by_name("a", "1"))
-    assert(Gem::Specification.find_by_name("a", ">1"))
-    assert_raises(Gem::LoadError) do
-      Gem::Specification.find_by_name("monkeys")
+  def test_missing_extensions_eh
+    ext_spec
+
+    assert @ext.missing_extensions?
+
+    extconf_rb = File.join @ext.gem_dir, @ext.extensions.first
+    FileUtils.mkdir_p File.dirname extconf_rb
+
+    open extconf_rb, 'w' do |f|
+      f.write <<-'RUBY'
+        open 'Makefile', 'w' do |f|
+          f.puts "clean:\n\techo clean"
+          f.puts "default:\n\techo built"
+          f.puts "install:\n\techo installed"
+        end
+      RUBY
     end
+
+    @ext.build_extensions
+
+    refute @ext.missing_extensions?
+  end
+
+  def test_missing_extensions_eh_default_gem
+    spec = new_default_spec 'default', 1
+    spec.extensions << 'extconf.rb'
+
+    refute spec.missing_extensions?
+  end
+
+  def test_missing_extensions_eh_legacy
+    ext_spec
+
+    @ext.installed_by_version = v '2.2.0.preview.2'
+
+    assert @ext.missing_extensions?
+
+    @ext.installed_by_version = v '2.2.0.preview.1'
+
+    refute @ext.missing_extensions?
+  end
+
+  def test_missing_extensions_eh_none
+    refute @a1.missing_extensions?
+  end
+
+  def test_find_by_name
+    util_spec "a"
+
+    assert Gem::Specification.find_by_name "a"
+    assert Gem::Specification.find_by_name "a", "1"
+    assert Gem::Specification.find_by_name "a", ">1"
+
+    assert_raises Gem::LoadError do
+      Gem::Specification.find_by_name "monkeys"
+    end
+  end
+
+  def test_find_by_name_prerelease
+    b = util_spec "b", "2.a"
+
+    b.activate
+
+    assert Gem::Specification.find_by_name "b"
+
+    assert_raises Gem::LoadError do
+      Gem::Specification.find_by_name "b", "1"
+    end
+
+    assert Gem::Specification.find_by_name "b", ">1"
   end
 
   def test_find_by_path
@@ -2441,7 +3011,7 @@ end
   end
 
   def util_setup_deps
-    @gem = quick_spec "awesome", "1.0" do |awesome|
+    @gem = util_spec "awesome", "1.0" do |awesome|
       awesome.add_runtime_dependency "bonobo", []
       awesome.add_development_dependency "monkey", []
     end
@@ -2469,9 +3039,9 @@ end
 
   def with_syck
     begin
+      verbose, $VERBOSE = $VERBOSE, nil
       require "yaml"
       old_engine = YAML::ENGINE.yamler
-      verbose, $VERBOSE = $VERBOSE, nil
       YAML::ENGINE.yamler = 'syck'
       load 'rubygems/syck_hack.rb'
     rescue NameError
